@@ -13,6 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import logging
 import os
 import sys
@@ -27,6 +28,7 @@ from tripleo_common.actions import heat_capabilities
 from tripleo_common import constants
 from tripleo_common.image import kolla_builder
 from tripleo_common.utils import plan as plan_utils
+from tripleo_common.utils import swift as swiftutils
 
 
 LOG = logging.getLogger(__name__)
@@ -67,7 +69,8 @@ class PrepareContainerImageEnv(base.TripleOAction):
         params = default_image_params()
         swift = self.get_object_client(context)
         try:
-            swift.put_object(
+            swiftutils.put_object_string(
+                swift,
                 self.container,
                 constants.CONTAINER_DEFAULTS_ENVIRONMENT,
                 yaml.safe_dump(
@@ -100,8 +103,10 @@ class PrepareContainerImageParameters(base.TripleOAction):
 
     def _get_role_data(self, swift):
         try:
-            j2_role_file = swift.get_object(
-                self.container, constants.OVERCLOUD_J2_ROLES_NAME)[1]
+            j2_role_file = swiftutils.get_object_string(
+                swift,
+                self.container,
+                constants.OVERCLOUD_J2_ROLES_NAME)
             role_data = yaml.safe_load(j2_role_file)
         except swiftexceptions.ClientException:
             LOG.info("No %s file found, not filtering container images by role"
@@ -146,7 +151,8 @@ class PrepareContainerImageParameters(base.TripleOAction):
                 os.remove(f)
 
         try:
-            swift.put_object(
+            swiftutils.put_object_string(
+                swift,
                 self.container,
                 constants.CONTAINER_DEFAULTS_ENVIRONMENT,
                 yaml.safe_dump(
@@ -165,3 +171,40 @@ class PrepareContainerImageParameters(base.TripleOAction):
         update_action = heat_capabilities.UpdateCapabilitiesAction(
             environments, container=self.container)
         return update_action.run(context)
+
+
+class ContainerImagePrepareDefault(base.TripleOAction):
+    """ContainerImagePrepare default parameters
+
+    """
+
+    def __init__(self, values):
+
+        super(ContainerImagePrepareDefault, self).__init__()
+        self.values = values
+
+    def run(self, context):
+        cip = copy.deepcopy(kolla_builder.CONTAINER_IMAGE_PREPARE_PARAM)
+
+        for entry in cip:
+            if 'push_destination' in self.values:
+                entry['push_destination'] = self.values['push_destination']
+
+            if 'tag_from_label' in self.values:
+                entry['tag_from_label'] = self.values['tag_from_label']
+
+            if 'namespace' in self.values:
+                entry['set']['namespace'] = self.values['namespace']
+
+            if 'name_prefix' in self.values:
+                entry['set']['name_prefix'] = self.values['name_prefix']
+
+            if 'name_suffix' in self.values:
+                entry['set']['name_suffix'] = self.values['name_suffix']
+
+            if 'tag' in self.values:
+                entry['set']['tag'] = self.values['tag']
+
+        return {
+            'ContainerImagePrepare': cip
+        }
