@@ -109,7 +109,7 @@ class ImageUploadManager(BaseImageManager):
 
     def __init__(self, config_files=None,
                  dry_run=False, cleanup=CLEANUP_FULL,
-                 mirrors=None):
+                 mirrors=None, registry_credentials=None):
         if config_files is None:
             config_files = []
         super(ImageUploadManager, self).__init__(config_files)
@@ -122,6 +122,26 @@ class ImageUploadManager(BaseImageManager):
         if mirrors:
             for uploader in self.uploaders.values():
                 uploader.mirrors.update(mirrors)
+        if registry_credentials:
+            self.validate_registry_credentials(registry_credentials)
+            for uploader in self.uploaders.values():
+                uploader.registry_credentials = registry_credentials
+
+    def validate_registry_credentials(self, creds_data):
+        if not isinstance(creds_data, dict):
+            raise TypeError('Credentials data must be a dict')
+        for registry, cred_entry in creds_data.items():
+            if not isinstance(cred_entry, dict) or len(cred_entry) != 1:
+                raise TypeError('Credentials entry must be'
+                                'a dict with a single item')
+            if not isinstance(registry, six.string_types):
+                raise TypeError('Key must be a registry host string: %s' %
+                                registry)
+            username, password = next(iter(cred_entry.items()))
+            if not (isinstance(username, six.string_types) and
+                    isinstance(password, six.string_types)):
+                raise TypeError('Username and password must be strings: %s' %
+                                username)
 
     def discover_image_tag(self, image, tag_from_label=None,
                            username=None, password=None):
@@ -197,6 +217,7 @@ class BaseImageUploader(object):
         # A mapping of layer hashs to the image which first copied that
         # layer to the target
         self.image_layers = {}
+        self.registry_credentials = {}
 
     @classmethod
     def init_registries_cache(cls):
@@ -215,6 +236,13 @@ class BaseImageUploader(object):
     def run_tasks(self):
         pass
 
+    def credentials_for_registry(self, registry):
+        creds = self.registry_credentials.get(registry)
+        if not creds:
+            return None, None
+        username, password = next(iter(creds.items()))
+        return username, password
+
     @classmethod
     def run_modify_playbook(cls, modify_role, modify_vars,
                             source_image, target_image, append_tag,
@@ -406,9 +434,18 @@ class BaseImageUploader(object):
         manifest_r.raise_for_status()
         tags_r.raise_for_status()
 
-        manifest = manifest_r.json()
+        manifest_str = manifest_r.text
+
+        if 'Docker-Content-Digest' in manifest_r.headers:
+            digest = manifest_r.headers['Docker-Content-Digest']
+        else:
+            # The registry didn't supply the manifest digest, so calculate it
+            calc_digest = hashlib.sha256()
+            calc_digest.update(manifest_str.encode('utf-8'))
+            digest = 'sha256:%s' % calc_digest.hexdigest()
+
+        manifest = json.loads(manifest_str)
 
-        digest = manifest_r.headers['Docker-Content-Digest']
         if manifest.get('schemaVersion', 2) == 1:
             config = json.loads(manifest['history'][0]['v1Compatibility'])
             layers = list(reversed([l['blobSum']
@@ -698,8 +735,13 @@ class SkopeoImageUploader(BaseImageUploader):
         if t.dry_run:
             return []
 
+        target_username, target_password = self.credentials_for_registry(
+            t.target_image_url.netloc)
         target_session = self.authenticate(
-            t.target_image_url)
+            t.target_image_url,
+            username=target_username,
+            password=target_password
+        )
 
         if t.modify_role and self._image_exists(
                 t.target_image, target_session):
@@ -707,8 +749,13 @@ class SkopeoImageUploader(BaseImageUploader):
                         t.target_image)
             return []
 
+        source_username, source_password = self.credentials_for_registry(
+            t.source_image_url.netloc)
         source_session = self.authenticate(
-            t.source_image_url)
+            t.source_image_url,
+            username=source_username,
+            password=source_password
+        )
 
         source_inspect = self._inspect(
             t.source_image_url,
@@ -858,8 +905,12 @@ class PythonImageUploader(BaseImageUploader):
         if t.dry_run:
             return []
 
+        target_username, target_password = self.credentials_for_registry(
+            t.target_image_url.netloc)
         target_session = self.authenticate(
-            t.target_image_url
+            t.target_image_url,
+            username=target_username,
+            password=target_password
         )
 
         self._detect_target_export(t.target_image_url, target_session)
@@ -874,8 +925,12 @@ class PythonImageUploader(BaseImageUploader):
         else:
             copy_target_url = t.target_image_url
 
+        source_username, source_password = self.credentials_for_registry(
+            t.source_image_url.netloc)
         source_session = self.authenticate(
-            t.source_image_url
+            t.source_image_url,
+            username=source_username,
+            password=source_password
         )
 
         manifest_str = self._fetch_manifest(
@@ -883,7 +938,11 @@ class PythonImageUploader(BaseImageUploader):
             session=source_session
         )
         manifest = json.loads(manifest_str)
-        source_layers = [l['digest'] for l in manifest['layers']]
+        if manifest.get('schemaVersion', 2) == 1:
+            source_layers = list(reversed([l['blobSum']
+                                          for l in manifest['fsLayers']]))
+        else:
+            source_layers = [l['digest'] for l in manifest['layers']]
 
         self._cross_repo_mount(
             copy_target_url, self.image_layers, source_layers,
@@ -1225,7 +1284,7 @@ class PythonImageUploader(BaseImageUploader):
         cls._assert_scheme(source_url, 'docker')
         pull_source = source_url.netloc + source_url.path
         LOG.info('Pulling %s' % pull_source)
-        cmd = ['podman', 'pull']
+        cmd = ['buildah', 'pull']
 
         if source_url.netloc in [cls.insecure_registries,
                                  cls.no_verify_registries]:
@@ -1536,7 +1595,7 @@ class PythonImageUploader(BaseImageUploader):
         if image_url.scheme != 'containers-storage':
             raise ImageUploaderException('Delete not implemented for %s' %
                                          image_url.geturl())
-        cmd = ['podman', 'rmi', image_url.path]
+        cmd = ['buildah', 'rmi', image_url.path]
         LOG.info('Running %s' % ' '.join(cmd))
         env = os.environ.copy()
         process = subprocess.Popen(cmd, env=env, stdout=subprocess.PIPE,
@@ -1630,7 +1689,9 @@ def upload_task(args):
 def discover_tag_from_inspect(args):
     self, image, tag_from_label = args
     image_url = self._image_to_url(image)
-    session = self.authenticate(image_url)
+    username, password = self.credentials_for_registry(image_url.netloc)
+    session = self.authenticate(
+        image_url, username=username, password=password)
     i = self._inspect(image_url, session=session)
     if ':' in image_url.path:
         # break out the tag from the url to be the fallback tag
