@@ -154,6 +154,69 @@ class TestImageUploadManager(base.TestCase):
                           manager.get_uploader,
                           'unknown')
 
+    def test_validate_registry_credentials(self):
+        # valid credentials
+        image_uploader.ImageUploadManager(
+            self.filelist,
+            registry_credentials=None)
+        image_uploader.ImageUploadManager(
+            self.filelist,
+            registry_credentials={})
+        manager = image_uploader.ImageUploadManager(
+            self.filelist,
+            registry_credentials={
+                'docker.io': {'my_username': 'my_password'},
+                u'quay.io': {u'quay_username': u'quay_password'},
+            })
+        self.assertEqual(
+            ('my_username', 'my_password'),
+            manager.uploader('python').credentials_for_registry('docker.io')
+        )
+        self.assertEqual(
+            ('quay_username', 'quay_password'),
+            manager.uploader('python').credentials_for_registry('quay.io')
+        )
+
+        # invalid credentials
+        self.assertRaises(
+            TypeError,
+            image_uploader.ImageUploadManager,
+            self.filelist,
+            registry_credentials='foo'
+        )
+        self.assertRaises(
+            TypeError,
+            image_uploader.ImageUploadManager,
+            self.filelist,
+            registry_credentials={
+                1234: {'my_username': 'my_password'},
+            }
+        )
+        self.assertRaises(
+            TypeError,
+            image_uploader.ImageUploadManager,
+            self.filelist,
+            registry_credentials={
+                'docker.io': {True: 'my_password'},
+            }
+        )
+        self.assertRaises(
+            TypeError,
+            image_uploader.ImageUploadManager,
+            self.filelist,
+            registry_credentials={
+                'docker.io': {'my_username': True},
+            }
+        )
+        self.assertRaises(
+            TypeError,
+            image_uploader.ImageUploadManager,
+            self.filelist,
+            registry_credentials={
+                'docker.io': {'my_username': 'my_password', 'foo': 'bar'},
+            }
+        )
+
 
 class TestBaseImageUploader(base.TestCase):
 
@@ -489,6 +552,7 @@ class TestBaseImageUploader(base.TestCase):
                 {'digest': 'ccc'},
             ]
         }
+        manifest_str = json.dumps(manifest_resp, indent=3)
         manifest_headers = {'Docker-Content-Digest': 'eeeeee'}
         tags_resp = {'tags': ['one', 'two', 'latest']}
         config_resp = {
@@ -518,7 +582,7 @@ class TestBaseImageUploader(base.TestCase):
 
         # test full response
         req.get('https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
-                json=manifest_resp, headers=manifest_headers)
+                text=manifest_str, headers=manifest_headers)
 
         self.assertEqual(
             {
@@ -576,6 +640,7 @@ class TestBaseImageUploader(base.TestCase):
                 {'blobSum': 'aaa'},
             ]
         }
+        manifest_str = json.dumps(manifest_resp, indent=3)
         manifest_headers = {'Docker-Content-Digest': 'eeeeee'}
         tags_resp = {'tags': ['one', 'two', 'latest']}
 
@@ -590,7 +655,7 @@ class TestBaseImageUploader(base.TestCase):
 
         # test full response
         req.get('https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
-                json=manifest_resp, headers=manifest_headers)
+                text=manifest_str, headers=manifest_headers)
 
         self.assertDictEqual(
             {
@@ -612,6 +677,82 @@ class TestBaseImageUploader(base.TestCase):
             inspect(url1, session=session)
         )
 
+    def test_inspect_no_digest_header(self):
+        req = self.requests
+        session = requests.Session()
+        session.headers['Authorization'] = 'Bearer asdf1234'
+        inspect = image_uploader.BaseImageUploader._inspect
+
+        url1 = urlparse('docker://docker.io/t/nova-api:latest')
+
+        manifest_resp = {
+            'schemaVersion': 2,
+            'config': {
+                'mediaType': 'text/html',
+                'digest': 'abcdef'
+            },
+            'layers': [
+                {'digest': 'aaa'},
+                {'digest': 'bbb'},
+                {'digest': 'ccc'},
+            ]
+        }
+        manifest_str = json.dumps(manifest_resp, indent=3)
+        manifest_headers = {}
+        tags_resp = {'tags': ['one', 'two', 'latest']}
+        config_resp = {
+            'created': '2018-10-02T11:13:45.567533229Z',
+            'docker_version': '1.13.1',
+            'config': {
+                'Labels': {
+                    'build-date': '20181002',
+                    'build_id': '1538477701',
+                    'kolla_version': '7.0.0'
+                }
+            },
+            'architecture': 'amd64',
+            'os': 'linux',
+        }
+
+        req.get('https://registry-1.docker.io/v2/t/nova-api/tags/list',
+                json=tags_resp)
+        req.get('https://registry-1.docker.io/v2/t/nova-api/blobs/abcdef',
+                json=config_resp)
+
+        # test 404 response
+        req.get('https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
+                status_code=404)
+        self.assertRaises(ImageNotFoundException, inspect, url1,
+                          session=session)
+
+        # test full response
+        req.get('https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
+                text=manifest_str, headers=manifest_headers)
+
+        calc_digest = hashlib.sha256()
+        calc_digest.update(manifest_str.encode('utf-8'))
+        digest = 'sha256:%s' % calc_digest.hexdigest()
+
+        self.assertEqual(
+            {
+                'Architecture': 'amd64',
+                'Created': '2018-10-02T11:13:45.567533229Z',
+                'Digest': digest,
+                'DockerVersion': '1.13.1',
+                'Labels': {
+                    'build-date': '20181002',
+                    'build_id': '1538477701',
+                    'kolla_version': '7.0.0'
+                },
+                'Layers': ['aaa', 'bbb', 'ccc'],
+                'Name': 'docker.io/t/nova-api',
+                'Os': 'linux',
+                'RepoTags': ['one', 'two', 'latest'],
+                'Tag': 'latest'
+            },
+            inspect(url1, session=session)
+        )
+
     @mock.patch('concurrent.futures.ThreadPoolExecutor')
     def test_list(self, mock_pool):
         mock_pool.return_value.map.return_value = (
@@ -1022,10 +1163,14 @@ class TestPythonImageUploader(base.TestCase):
 
         authenticate.assert_has_calls([
             mock.call(
-                target_url
+                target_url,
+                username=None,
+                password=None
             ),
             mock.call(
-                source_url
+                source_url,
+                username=None,
+                password=None
             ),
         ])
 
@@ -1050,6 +1195,77 @@ class TestPythonImageUploader(base.TestCase):
             target_session=target_session
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.authenticate')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._fetch_manifest')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._cross_repo_mount')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._copy_registry_to_registry')
+    def test_authenticate_upload_image(
+            self, _copy_registry_to_registry, _cross_repo_mount,
+            _fetch_manifest, authenticate):
+
+        self.uploader.registry_credentials = {
+            'docker.io': {'my_username': 'my_password'},
+            'localhost:8787': {'local_username': 'local_password'},
+        }
+        target_session = mock.Mock()
+        source_session = mock.Mock()
+        authenticate.side_effect = [
+            target_session,
+            source_session
+        ]
+        manifest = json.dumps({
+            'config': {
+                'digest': 'sha256:1234',
+            },
+            'layers': [
+                {'digest': 'sha256:aaa'},
+                {'digest': 'sha256:bbb'},
+                {'digest': 'sha256:ccc'}
+            ],
+        })
+        _fetch_manifest.return_value = manifest
+
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
+        tag = 'latest'
+        push_destination = 'localhost:8787'
+        # push_image = 'localhost:8787/tripleomaster/heat-docker-agents-centos'
+        task = image_uploader.UploadTask(
+            image_name=image + ':' + tag,
+            pull_source=None,
+            push_destination=push_destination,
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup='full'
+        )
+
+        self.assertEqual(
+            [],
+            self.uploader.upload_image(task)
+        )
+        source_url = urlparse('docker://docker.io/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+        target_url = urlparse('docker://localhost:8787/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+
+        authenticate.assert_has_calls([
+            mock.call(
+                target_url,
+                username='local_username',
+                password='local_password'
+            ),
+            mock.call(
+                source_url,
+                username='my_username',
+                password='my_password'
+            ),
+        ])
+
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1105,13 +1321,103 @@ class TestPythonImageUploader(base.TestCase):
 
         authenticate.assert_has_calls([
             mock.call(
-                target_url
+                target_url,
+                username=None,
+                password=None
             ),
             mock.call(
-                source_url
+                source_url,
+                username=None,
+                password=None
             ),
         ])
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.authenticate')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._fetch_manifest')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._cross_repo_mount')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._copy_registry_to_registry')
+    def test_upload_image_v1_manifest(
+            self, _copy_registry_to_registry, _cross_repo_mount,
+            _fetch_manifest, authenticate):
+
+        target_session = mock.Mock()
+        source_session = mock.Mock()
+        authenticate.side_effect = [
+            target_session,
+            source_session
+        ]
+        manifest = json.dumps({
+            'schemaVersion': 1,
+            'fsLayers': [
+                {'blobSum': 'sha256:ccc'},
+                {'blobSum': 'sha256:bbb'},
+                {'blobSum': 'sha256:aaa'}
+            ],
+        })
+        _fetch_manifest.return_value = manifest
+
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
+        tag = 'latest'
+        push_destination = 'localhost:8787'
+        # push_image = 'localhost:8787/tripleomaster/heat-docker-agents-centos'
+        task = image_uploader.UploadTask(
+            image_name=image + ':' + tag,
+            pull_source=None,
+            push_destination=push_destination,
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup='full'
+        )
+
+        self.assertEqual(
+            [],
+            self.uploader.upload_image(task)
+        )
+        source_url = urlparse('docker://docker.io/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+        target_url = urlparse('docker://localhost:8787/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+
+        authenticate.assert_has_calls([
+            mock.call(
+                target_url,
+                username=None,
+                password=None
+            ),
+            mock.call(
+                source_url,
+                username=None,
+                password=None
+            ),
+        ])
+
+        _fetch_manifest.assert_called_once_with(
+            source_url, session=source_session)
+
+        _cross_repo_mount.assert_called_once_with(
+            target_url,
+            {
+                'sha256:aaa': target_url,
+                'sha256:bbb': target_url,
+                'sha256:ccc': target_url,
+            },
+            ['sha256:aaa', 'sha256:bbb', 'sha256:ccc'],
+            session=target_session)
+
+        _copy_registry_to_registry.assert_called_once_with(
+            source_url,
+            target_url,
+            source_manifest=manifest,
+            source_session=source_session,
+            target_session=target_session
+        )
+
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1190,10 +1496,14 @@ class TestPythonImageUploader(base.TestCase):
         )
         authenticate.assert_has_calls([
             mock.call(
-                target_url
+                target_url,
+                username=None,
+                password=None
             ),
             mock.call(
-                source_url
+                source_url,
+                username=None,
+                password=None
             ),
         ])
 
@@ -1842,7 +2152,7 @@ class TestPythonImageUploader(base.TestCase):
             self.uploader._delete(url)
         )
         mock_popen.assert_called_once_with([
-            'podman',
+            'buildah',
             'rmi',
             '/t/nova-api:latest'],
             env={}, stdout=-1,
